/*
 * src/instance.h
 *
 * Copyright (C) 2019 Ilja Kartašov <ik@lowenware.com>
 *
 * Project homepage: https://lowenware.com/aisl/
 *
 */

#ifndef AISL_INSTANCE_H_814CF474_A646_45B7_B6B2_3F4C7BEFA484
#define AISL_INSTANCE_H_814CF474_A646_45B7_B6B2_3F4C7BEFA484

#ifndef AISL_WITHOUT_SSL
#include <openssl/ssl.h>
#endif

#include <aisl/instance.h>
#include "list.h"


struct aisl
{
  struct list  server_spool;
  struct list  client_spool;
  #ifndef AISL_WITHOUT_SSL
  struct list  ssl_spool;
  #endif

  char       * last_error;

  int32_t      iterator;

  uint32_t     accept_limit;
  uint32_t     silence_timeout;

  /* callbacks */
  aisl_on_server_open_t          on_server_open;
  aisl_on_server_error_t         on_server_error;
  aisl_on_client_connect_t       on_client_connect;
  aisl_on_client_disconnect_t    on_client_disconnect;
  aisl_on_client_timeout_t       on_client_timeout;
  aisl_on_stream_open_t          on_stream_open;
  aisl_on_stream_header_t        on_stream_header;
  aisl_on_stream_input_t         on_stream_input;
  aisl_on_stream_request_t       on_stream_request;
  aisl_on_stream_output_t        on_stream_output;
  aisl_on_stream_close_t         on_stream_close;
  aisl_on_stream_error_t         on_stream_error;
};


#ifndef AISL_WITHOUT_SSL
/**
 * @brief Gets SSL context for appropriate server name.
 * @param instance a pointer to #aisl_t instance.
 * @param server_name a null-terminated string with server name or NULL.
 * @return a pointer to SSL context
 */
SSL_CTX *
aisl_get_ssl_ctx( aisl_t instance, const char * server_name );
#endif


/**
 * @brief Sets error message for AISL instace.
 * @param instance a pointer to #aisl_t instance.
 * @param err_msge a null-terminated string with error message.
 */
void
aisl_set_error( aisl_t instance, const char * err_msg );


/** 
 * @brief Raises event from source.
 * @param instance a pointer to #aisl_t instance.
 * @param source a pointer to an event source.
 * @param event a code of event.
 * @param ... a list of arguments specific for event.
 */
void
aisl_raise( aisl_t instance, void * source, aisl_event_t event, ... );


#endif /* !AISL_INSTANCE_H */
