/******************************************************************************
 *
 *                Copyright (c) 2017-2019 by Löwenware Ltd
 *             Please, refer LICENSE file for legal information
 *
 ******************************************************************************/

/**
 * @file types.c
 * @author Ilja Kartašov <ik@lowenware.com>
 * @brief Types stringifiers module
 *
 * @see https://lowenware.com/aisl/
 */

#include <aisl/types.h>


#ifndef WITHOUT_STRINGIFIERS

__attribute__ ((visibility ("default") ))
const char *
aisl_status_to_string(aisl_status_t status)
{
  switch(status)
  {
    case AISL_INPUT_ERROR:   return "AISL_INPUT_ERROR";
    case AISL_EXTCALL_ERROR: return "AISL_EXTCALL_ERROR";
    case AISL_SYSCALL_ERROR: return "AISL_SYSCALL_ERROR";
    case AISL_MALLOC_ERROR:  return "AISL_MALLOC_ERROR";
    case AISL_SUCCESS:       return "AISL_SUCCESS";
    case AISL_IDLE:          return "AISL_IDLE";
  }
  return "UNKNOWN";
}


__attribute__ ((visibility ("default") ))
const char *
aisl_event_to_string( aisl_event_t event )
{
  switch(event)
  {
    case AISL_EVENT_SERVER_OPEN:         return "SERVER_OPEN";
    case AISL_EVENT_SERVER_ERROR:        return "SERVER_ERROR";
    case AISL_EVENT_CLIENT_CONNECT:      return "CLIENT_CONNECT";
    case AISL_EVENT_CLIENT_DISCONNECT:   return "CLIENT_DISCONNECT";
    case AISL_EVENT_CLIENT_TIMEOUT:      return "CLIENT_TIMEOUT";
    case AISL_EVENT_STREAM_OPEN:         return "STREAM_OPEN";
    case AISL_EVENT_STREAM_HEADER:       return "STREAM_HEADER";
    case AISL_EVENT_STREAM_INPUT:        return "STREAM_INPUT";
    case AISL_EVENT_STREAM_REQUEST:      return "STREAM_REQUEST";
    case AISL_EVENT_STREAM_OUTPUT:       return "STREAM_OUTPUT";
    case AISL_EVENT_STREAM_CLOSE:        return "STREAM_CLOSE";
    case AISL_EVENT_STREAM_ERROR:        return "STREAM_ERROR";
  }

  return "UNKNOWN";
}

#endif
